# The Redox Operating-System

[Introduction](./introduction.md)

# Introduction and Getting Started

- [Introducing Redox](./introducing-redox.md)
  - [Our Goals](./our-goals.md)
  - [Our Philosophy](./philosophy.md)
  - [Why a New OS?](./why-a-new-os.md)
  - [Redox Use Cases](./redox-use-cases.md)
  - [Comparing Redox to Other OSes](./how-redox-compares.md)
  - [Why Rust?](./why-rust.md)
  - [Side Projects](./side-projects.md)
  - [Influences](./influences.md)
  - [Hardware Support](./hardware-support.md)
  - [Important Programs](./important-programs.md)

- [Getting started](./getting-started.md)
  - [Running Redox in a Virtual Machine](./running-vm.md)
  - [Running Redox on Real Hardware](./real-hardware.md)
  - [Installing Redox on a Drive](./installing.md)
  - [Trying Out Redox](./trying-out-redox.md)
  - [Building Redox](./podman-build.md)
  - [Native Build](./building-redox.md)
  - [From Nothing To Hello World](./nothing-to-hello-world.md)
  - [Configuration Settings](./configuration-settings.md)
  - [Package Management](./package-management.md)
  - [Downloading Packages with pkg](./pkg.md)
  - [Tasks](./tasks.md)
  - [Questions and Feedback](./asking-questions.md)

# Architecture and Design

- [System Design](./system-design.md)
  - [Microkernels](./microkernels.md)
  - [Boot Process](./boot-process.md)
  - [Redox kernel](./kernel.md)
  - [System Services in User Space](./user-space.md)
  - [Communication](./communication.md)
  - [Memory Management](./memory.md)
  - [Scheduling](./scheduling.md)
  - [Drivers](./drivers.md)
  - [RedoxFS](./redoxfs.md)
  - [Graphics and Windowing](./graphics-windowing.md)
  - [Security](./security.md)
  - [Features](./features.md)

- [Schemes and Resources](./schemes-resources.md)
  - [Scheme-rooted Paths](./scheme-rooted-paths.md)
  - [Resources](./resources.md)
  - [Schemes](./schemes.md)
  - ["Everything is a File"](./everything-is-a-file.md)
  - [Stitching It All Together](./stitching-it-all-together.md)
  - [Scheme Operation](./scheme-operation.md)
  - [Event Scheme](./event-scheme.md)
  - [An Example](./example.md)

- [Programs and Libraries](./programs-libraries.md)
  - [Components](./components.md)
  - [GUI](./gui.md)
  - [Shell](./shell.md)
  - [System Tools](./system-tools.md)

# Developing with and for Redox

- [The Redox Build Process](./build-process.md)
  - [Advanced Podman Build](./advanced-podman-build.md)
  - [Advanced Native Build](./advanced-build.md)
  - [Working with i686](./i686.md)
  - [Working with ARM64](./aarch64.md)
  - [Raspberry Pi](./raspi.md)
  - [Troubleshooting the Build](./troubleshooting.md)
  - [Build System](./build-system-reference.md)
  - [Build Phases](./build-phases.md)

- [Developing for Redox](./developing-for-redox.md)
  - [Including Programs in Redox](./including-programs.md)
  - [Coding and Building](./coding-and-building.md)
  - [Porting Applications using Recipes](./porting-applications.md)
  - [Porting Case Study](./porting-case-study.md)
  - [Quick Workflow](./quick-workflow.md)
  - [Libraries and APIs](./libraries-apis.md)
  - [Developer FAQ](./developer-faq.md)
  - [References](./references.md)
  - [Continuous Integration](./ci.md)
  - [Performance](./performance.md)
  - [System Call Tracing](./syscall-debug.md)

# Contributing

- [Contributing](./contributing.md)
  - [Chat](./chat.md)

- [Best practices and guidelines](./best-practices.md)
  - [Literate programming](./literate-programming.md)
  - [Writing docs correctly](./writing-docs-correctly.md)
  - [Style](./style.md)
  - [Rusting properly](./rusting-properly.md)
  - [Avoiding Panics](./avoiding-panics.md)
  - [Testing Practices](./testing-practices.md)

- [Using Redox GitLab](./using-redox-gitlab.md)
  - [Signing in to GitLab](./signing-in-to-gitlab.md)
  - [Repository Structure](./repository-structure.md)
  - [Creating Proper Bug Reports](./creating-proper-bug-reports.md)
  - [Creating Proper Pull Requests](./creating-proper-pull-requests.md)
  - [Filing Issues](./filing-issues.md)
